// Created by John Liaperdos and George Theophanous, July 2015

var RE_values=[0,100,500,1000,1500,2000,5000]; // in Ohms
var RE_index=0;
var beta_values=[40,50,60,70,80,90,100,110,120,130,140,150,160];
var beta_index=6;
var IC,VC;
var VB,IB;
var VE;
var RE_colors=['#008000', '#800000', '#000000', '#0000ff', '#ff6600', '#666666', '#808000' ];
// plot area rectangle coordinates
	var screen_x_min=66.670074;
	var screen_y_min=390+37;
	var screen_x_max=373.32263;
	var screen_y_max=541.5+37;
// plot ranges
	var range_x_min=40;
	var range_x_max=170;
	var range_y_min=0;
	var range_y_max=10;

var measurements = new Array(7); //RE indices (0-6)
for (var i = 0; i < 7; i++) 
{
	measurements[i] = new Array(13); //beta values (40-160)
	for (var j = 0; j < 13; j++)
	{
		measurements[i][j] = -1; //IC : "-1" indicates that no measurement has been added
	}
}

function initializeBJTbiasing(){
	for(i=0;i<7;i++)
	{
		document.getElementById('RE_label_'+i).style.display='none';
		document.getElementById('RE_label_'+i).textContent=RE_values[i]+'Ω';
	}
	document.getElementById('message_text').textContent='';
	document.getElementById('RE_text').textContent='0Ω';
	document.getElementById('RE_text').style.fill=RE_colors[0];
	document.getElementById('color_indicator').style.fill=RE_colors[0];
	showResults(RE_values[RE_index],beta_values[beta_index]);	
}

function showResults(RE_val,beta_val){
	IC=CollectorCurrent(RE_val,beta_val);
	IB=BaseCurrent(IC,beta_val);
	VC=CollectorVoltage(RE_val,beta_val);
	VE=EmitterVoltage(IC,RE_val);
	VB=BaseVoltage(IC,RE_val);
	IC=Math.round(100000*IC)/100;
	IB=Math.round(100000000*IB)/100;
	VC=Math.round(100*VC)/100;
	VE=Math.round(100*VE)/100;
	VB=Math.round(100*VB)/100;
	document.getElementById('IC_text').textContent=IC;	
	document.getElementById('IB_text').textContent=IB;
	document.getElementById('VC_text').textContent=VC;
	document.getElementById('VE_text').textContent=VE;
	document.getElementById('VB_text').textContent=VB;
	document.getElementById('message_text').textContent='';	
}

function CollectorCurrent(RE_val,beta_val){
	collector_current=4.3*beta_val/(100000+beta_val*RE_val);
	return collector_current;
}

function BaseCurrent(IC_val,beta_val){
	base_current=IC_val/beta_val;
	return base_current;
}

function CollectorVoltage(RE_val,beta_val){
	collector_voltage=15-(4300*beta_val/(100000+beta_val*RE_val));
	return collector_voltage;
}

function EmitterVoltage(IC_val,RE_val){
	emitter_voltage=IC_val*RE_val;
	return emitter_voltage;
}

function BaseVoltage(IC_val,RE_val){
	base_voltage=0.7+IC_val*RE_val;
	return base_voltage;
}

function addPoint(){
	//check if point has been already added	
		if(measurements[RE_index][beta_index]!=-1)
		{
			document.getElementById('message_text').style.fill='red';
			document.getElementById('message_text').textContent='Η μέτρηση έχει ήδη προστεθεί';
		}
		else
		{
			measurements[RE_index][beta_index]=IC;
			document.getElementById('message_text').style.fill='green';
			document.getElementById('message_text').textContent='Η μέτρηση προστέθηκε';	
			document.getElementById('RE_label_'+RE_index).style.display='inline';
			plotPoint();
			// also plot line if enough points exist
				count_points=0;
				for (i=0;i<13;i++)
				{
					if(measurements[RE_index][i]>-1)
					{
						count_points=count_points+1;
					}
				}
				if(count_points==13)
				{
					connectPoints(RE_index);
				}
		}
}

function plotPoint(){
	var point_container=document.getElementById('RE'+RE_index+'_plot_container');
	var new_ellipse = document.createElementNS("http://www.w3.org/2000/svg", "ellipse");
	new_ellipse.setAttribute('cx', xformX(beta_values[beta_index]));
	new_ellipse.setAttribute('cy', xformY(IC));
	new_ellipse.setAttribute('rx', '3');
	new_ellipse.setAttribute('ry', '3');	
	new_ellipse.setAttribute('style', 'fill:'+RE_colors[RE_index]+';stroke:'+RE_colors[RE_index]+';stroke-width:2');
	point_container.appendChild(new_ellipse);
}

function clearPlots(){
	for(i=0;i<7;i++)
	{
		document.getElementById('RE_label_'+i).style.display='none';
		var point_container=document.getElementById('RE'+i+'_plot_container');
		while (point_container.hasChildNodes()) {
			point_container.removeChild(point_container.lastChild);
		}
		for (var j = 0; j < 13; j++)
		{
			measurements[i][j] = -1; 
		}
	}
	document.getElementById('message_text').textContent='';
}

function connectPoints(RE_index_val){
	if(RE_index_val==100) 
	{
		imin=0;
		imax=7;
	}
	else
	{
		imin=RE_index_val;
		imax=RE_index_val+1;
	}
	for(i=imin;i<imax;i++)
	{
		if(RE_index_val==100)
		{
			document.getElementById('RE_label_'+i).style.display='inline';
		};
		use_color=RE_colors[i];
		for (var j = beta_values[0]; j < beta_values[12]; j=j+0.5)
		{
			var plot_container=document.getElementById('RE'+i+'_plot_container');
			var new_line = document.createElementNS("http://www.w3.org/2000/svg", "line");
			new_line.setAttribute('x1', xformX(j));
			new_line.setAttribute('y1', xformY(1000*CollectorCurrent(RE_values[i],j)));
			new_line.setAttribute('x2', xformX(j+0.5));
			new_line.setAttribute('y2', xformY(1000*CollectorCurrent(RE_values[i],j)));	
			new_line.setAttribute('style', 'fill:'+RE_colors[i]+';stroke:'+RE_colors[i]+';stroke-width:2');
			plot_container.appendChild(new_line);			
		}
	}
	document.getElementById('message_text').textContent='';
}

function increaseRE(){
	if(RE_index<6)
	{
		RE_index=RE_index+1;
	}
	document.getElementById('RE_text').textContent=RE_values[RE_index]+'Ω';
	document.getElementById('RE_text').style.fill=RE_colors[RE_index];
	document.getElementById('color_indicator').style.fill=RE_colors[RE_index];
	showResults(RE_values[RE_index],beta_values[beta_index]);
}

function decreaseRE(){
	if(RE_index>0)
	{
		RE_index=RE_index-1;
	}
	document.getElementById('RE_text').textContent=RE_values[RE_index]+'Ω';
	document.getElementById('RE_text').style.fill=RE_colors[RE_index];
	document.getElementById('color_indicator').style.fill=RE_colors[RE_index];
	showResults(RE_values[RE_index],beta_values[beta_index]);
}

function increaseBeta(){
	if(beta_index<12)
	{
		beta_index=beta_index+1;
	}
	document.getElementById('beta_text').textContent='β='+beta_values[beta_index];
	showResults(RE_values[RE_index],beta_values[beta_index]);
}

function decreaseBeta(){
	if(beta_index>0)
	{
		beta_index=beta_index-1;
	}
	document.getElementById('beta_text').textContent='β='+beta_values[beta_index];
	showResults(RE_values[RE_index],beta_values[beta_index]);
}

function xformX(x_val){
	x_transformed=screen_x_min+(x_val-range_x_min)*(screen_x_max-screen_x_min)/(range_x_max-range_x_min);
	return x_transformed;
}

function xformY(y_val){
	y_transformed=screen_y_max-(y_val-range_y_min)*(screen_y_max-screen_y_min)/(range_y_max-range_y_min);
	return y_transformed;
}


